/*
 * xreboot version 1.0
 * 
 * Simple two-button application to reboot or shutdown a workstation
 * from the xdm login screen.
 * 
 * Copyright (c) 1997 Brendan O'Dea <bod@compusol.com.au>
 * 
 * Permission is hereby granted, free of charge, to any person obtain-
 * ing a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute,  sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so,  subject
 * to the following conditions:
 *
 * The  above  copyright  notice  and  this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT  WARRANTY  OF  ANY  KIND,
 * EXPRESS  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND  NONINFRINGE-
 * MENT.   IN  NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAM-
 * AGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT  OR
 * OTHERWISE,  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <unistd.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>

#if defined(SYSV) || defined(SVR4)
# define CMD_PATH	"/etc/shutdown"
# define CMD_ARGS_PRE	"shutdown"
# define CMD_ARG_REBOOT	"-i6"
# define CMD_ARG_HALT	"-i0"
# define CMD_ARGS_POST	"-g0", "-y"
#else /* SYSV || SVR4 */
# define CMD_PATH	"/sbin/shutdown"
# define CMD_ARGS_PRE	"shutdown"
# define CMD_ARG_REBOOT	"-r"
# define CMD_ARG_HALT	"-h"
# define CMD_ARGS_POST	"now"
#endif /* SYSV || SVR4 */

#ifdef __GNUC__
# define ATTR_UNUSED __attribute__ ((unused))
#else /* __GNUC__ */
# define ATTR_UNUSED
#endif /* __GNUC__ */

/*ARGSUSED*/
static void pushed(Widget widget ATTR_UNUSED, XtPointer closure,
		   XtPointer data ATTR_UNUSED)
{
    execl(CMD_PATH, CMD_ARGS_PRE, (char *) closure, CMD_ARGS_POST, 0);
    exit(1);
}

static String fallback_res[] = {
    "*geometry: +0-0",
    0
};

#include "reboot.xbm"
#include "halt.xbm"

int main(int argc, char *argv[])
{
    XtAppContext app;
    Widget shell = XtVaAppInitialize(
	&app, "XReboot", 0, 0, &argc, argv, fallback_res,
	0
    );

    Display *dpy = XtDisplay(shell);
    Window root = DefaultRootWindow(dpy);

    /* create button box */
    Widget buttonbox = XtVaCreateManagedWidget(
	"buttonbox", boxWidgetClass, shell,
	0
    );

    /* cursor */
    Cursor cursor = XCreateFontCursor(dpy, XC_left_ptr);

    /* create reboot button */
    Widget reboot = XtVaCreateManagedWidget(
	"reboot", commandWidgetClass, buttonbox,
	XtNlabel, "Re-boot",
	XtNcursor, cursor,
	XtNleftBitmap, XCreateBitmapFromData(
	    dpy, root, (char *) reboot_bits, reboot_width, reboot_height
	),
	0
    );

    /* create halt button */
    Widget halt = XtVaCreateManagedWidget(
	"halt", commandWidgetClass, buttonbox,
	XtNlabel, "Shutdown",
	XtNcursor, cursor,
	XtNleftBitmap, XCreateBitmapFromData(
	    dpy, root, (char *) halt_bits, halt_width, halt_height
	),
	0
    );

    /* add callbacks */
    XtAddCallback(reboot, XtNcallback, pushed, CMD_ARG_REBOOT);
    XtAddCallback(halt, XtNcallback, pushed, CMD_ARG_HALT);

    /* ensure that we are not holding any mounted dirs open */
    chdir("/");

    /* go for it */
    XtRealizeWidget(shell);
    XtAppMainLoop(app);

    /*NOTREACHED*/
    return 0;
}
